@@ -328,6 +328,11 @@ class Device(ABC):
         """Send a raw query to the device."""
         return await self.protocol.query(request=request)
 
+    @property
+    def parent(self) -> Device | None:
+        """Return the parent on child devices."""
+        return self._parent
+
     @property
     def children(self) -> Sequence[Device]:
         """Returns the child devices."""
