@@ -193,11 +193,9 @@ class SmartDevice(Device):
         if not self._features:
             await self._initialize_features()
 
-        _LOGGER.debug(
-            "Update completed %s: %s",
-            self.host,
-            self._last_update if first_update else resp,
-        )
+        if _LOGGER.isEnabledFor(logging.DEBUG):
+            updated = self._last_update if first_update else resp
+            _LOGGER.debug("Update completed %s: %s", self.host, list(updated.keys()))
 
     def _handle_module_post_update_hook(self, module: SmartModule) -> bool:
         try:
