@@ -1,5 +1,8 @@
 # type: ignore
+# ruff: noqa: S106
+
 import asyncio
+import logging
 import re
 import socket
 from unittest.mock import MagicMock
@@ -10,26 +13,28 @@ from async_timeout import timeout as asyncio_timeout
 
 from kasa import (
     Credentials,
+    Device,
     DeviceType,
     Discover,
-    SmartDevice,
-    SmartDeviceException,
+    KasaException,
 )
 from kasa.deviceconfig import (
-    ConnectionType,
     DeviceConfig,
+    DeviceConnectionParameters,
 )
 from kasa.discover import DiscoveryResult, _DiscoverProtocol, json_dumps
-from kasa.exceptions import AuthenticationException, UnsupportedDeviceException
+from kasa.exceptions import AuthenticationError, UnsupportedDeviceError
+from kasa.iot import IotDevice
 from kasa.xortransport import XorEncryption
 
 from .conftest import (
     bulb_iot,
-    dimmer,
-    lightstrip,
+    dimmer_iot,
+    lightstrip_iot,
     new_discovery,
-    plug,
+    plug_iot,
     strip_iot,
+    wallswitch_iot,
 )
 
 UNSUPPORTED = {
@@ -54,15 +59,22 @@ UNSUPPORTED = {
 }
 
 
-@plug
-async def test_type_detection_plug(dev: SmartDevice):
+@wallswitch_iot
+async def test_type_detection_switch(dev: Device):
+    d = Discover._get_device_class(dev._last_update)("localhost")
+    assert d.is_wallswitch
+    assert d.device_type == DeviceType.WallSwitch
+
+
+@plug_iot
+async def test_type_detection_plug(dev: Device):
     d = Discover._get_device_class(dev._last_update)("localhost")
     assert d.is_plug
     assert d.device_type == DeviceType.Plug
 
 
 @bulb_iot
-async def test_type_detection_bulb(dev: SmartDevice):
+async def test_type_detection_bulb(dev: Device):
     d = Discover._get_device_class(dev._last_update)("localhost")
     # TODO: light_strip is a special case for now to force bulb tests on it
     if not d.is_light_strip:
@@ -71,21 +83,21 @@ async def test_type_detection_bulb(dev: SmartDevice):
 
 
 @strip_iot
-async def test_type_detection_strip(dev: SmartDevice):
+async def test_type_detection_strip(dev: Device):
     d = Discover._get_device_class(dev._last_update)("localhost")
     assert d.is_strip
     assert d.device_type == DeviceType.Strip
 
 
-@dimmer
-async def test_type_detection_dimmer(dev: SmartDevice):
+@dimmer_iot
+async def test_type_detection_dimmer(dev: Device):
     d = Discover._get_device_class(dev._last_update)("localhost")
     assert d.is_dimmer
     assert d.device_type == DeviceType.Dimmer
 
 
-@lightstrip
-async def test_type_detection_lightstrip(dev: SmartDevice):
+@lightstrip_iot
+async def test_type_detection_lightstrip(dev: Device):
     d = Discover._get_device_class(dev._last_update)("localhost")
     assert d.is_light_strip
     assert d.device_type == DeviceType.LightStrip
@@ -93,12 +105,11 @@ async def test_type_detection_lightstrip(dev: SmartDevice):
 
 async def test_type_unknown():
     invalid_info = {"system": {"get_sysinfo": {"type": "nosuchtype"}}}
-    with pytest.raises(UnsupportedDeviceException):
+    with pytest.raises(UnsupportedDeviceError):
         Discover._get_device_class(invalid_info)
 
 
 @pytest.mark.parametrize("custom_port", [123, None])
-# @pytest.mark.parametrize("discovery_mock", [("127.0.0.1",123), ("127.0.0.1",None)], indirect=True)
 async def test_discover_single(discovery_mock, custom_port, mocker):
     """Make sure that discover_single returns an initialized SmartDevice instance."""
     host = "127.0.0.1"
@@ -106,19 +117,21 @@ async def test_discover_single(discovery_mock, custom_port, mocker):
     discovery_mock.port_override = custom_port
 
     device_class = Discover._get_device_class(discovery_mock.discovery_data)
-    update_mock = mocker.patch.object(device_class, "update")
+    # discovery_mock patches protocol query methods so use spy here.
+    update_mock = mocker.spy(device_class, "update")
 
     x = await Discover.discover_single(
         host, port=custom_port, credentials=Credentials()
     )
-    assert issubclass(x.__class__, SmartDevice)
+    assert issubclass(x.__class__, Device)
     assert x._discovery_info is not None
     assert x.port == custom_port or x.port == discovery_mock.default_port
+    # Make sure discovery does not call update()
     assert update_mock.call_count == 0
     if discovery_mock.default_port == 80:
         assert x.alias is None
 
-    ct = ConnectionType.from_values(
+    ct = DeviceConnectionParameters.from_values(
         discovery_mock.device_type,
         discovery_mock.encrypt_type,
         discovery_mock.login_version,
@@ -144,23 +157,77 @@ async def test_discover_single_hostname(discovery_mock, mocker):
     update_mock = mocker.patch.object(device_class, "update")
 
     x = await Discover.discover_single(host, credentials=Credentials())
-    assert issubclass(x.__class__, SmartDevice)
+    assert issubclass(x.__class__, Device)
     assert x._discovery_info is not None
     assert x.host == host
     assert update_mock.call_count == 0
 
     mocker.patch("socket.getaddrinfo", side_effect=socket.gaierror())
-    with pytest.raises(SmartDeviceException):
+    with pytest.raises(KasaException):
         x = await Discover.discover_single(host, credentials=Credentials())
 
 
+async def test_discover_credentials(mocker):
+    """Make sure that discover gives credentials precedence over un and pw."""
+    host = "127.0.0.1"
+    mocker.patch("kasa.discover._DiscoverProtocol.wait_for_discovery_to_complete")
+
+    def mock_discover(self, *_, **__):
+        self.discovered_devices = {host: MagicMock()}
+
+    mocker.patch.object(_DiscoverProtocol, "do_discover", mock_discover)
+    dp = mocker.spy(_DiscoverProtocol, "__init__")
+
+    # Only credentials passed
+    await Discover.discover(credentials=Credentials(), timeout=0)
+    assert dp.mock_calls[0].kwargs["credentials"] == Credentials()
+    # Credentials and un/pw passed
+    await Discover.discover(
+        credentials=Credentials(), username="Foo", password="Bar", timeout=0
+    )
+    assert dp.mock_calls[1].kwargs["credentials"] == Credentials()
+    # Only un/pw passed
+    await Discover.discover(username="Foo", password="Bar", timeout=0)
+    assert dp.mock_calls[2].kwargs["credentials"] == Credentials("Foo", "Bar")
+    # Only un passed, credentials should be None
+    await Discover.discover(username="Foo", timeout=0)
+    assert dp.mock_calls[3].kwargs["credentials"] is None
+
+
+async def test_discover_single_credentials(mocker):
+    """Make sure that discover_single gives credentials precedence over un and pw."""
+    host = "127.0.0.1"
+    mocker.patch("kasa.discover._DiscoverProtocol.wait_for_discovery_to_complete")
+
+    def mock_discover(self, *_, **__):
+        self.discovered_devices = {host: MagicMock()}
+
+    mocker.patch.object(_DiscoverProtocol, "do_discover", mock_discover)
+    dp = mocker.spy(_DiscoverProtocol, "__init__")
+
+    # Only credentials passed
+    await Discover.discover_single(host, credentials=Credentials(), timeout=0)
+    assert dp.mock_calls[0].kwargs["credentials"] == Credentials()
+    # Credentials and un/pw passed
+    await Discover.discover_single(
+        host, credentials=Credentials(), username="Foo", password="Bar", timeout=0
+    )
+    assert dp.mock_calls[1].kwargs["credentials"] == Credentials()
+    # Only un/pw passed
+    await Discover.discover_single(host, username="Foo", password="Bar", timeout=0)
+    assert dp.mock_calls[2].kwargs["credentials"] == Credentials("Foo", "Bar")
+    # Only un passed, credentials should be None
+    await Discover.discover_single(host, username="Foo", timeout=0)
+    assert dp.mock_calls[3].kwargs["credentials"] is None
+
+
 async def test_discover_single_unsupported(unsupported_device_info, mocker):
     """Make sure that discover_single handles unsupported devices correctly."""
     host = "127.0.0.1"
 
     # Test with a valid unsupported response
     with pytest.raises(
-        UnsupportedDeviceException,
+        UnsupportedDeviceError,
     ):
         await Discover.discover_single(host)
 
@@ -170,7 +237,7 @@ async def test_discover_single_no_response(mocker):
     host = "127.0.0.1"
     mocker.patch.object(_DiscoverProtocol, "do_discover")
     with pytest.raises(
-        SmartDeviceException, match=f"Timed out getting discovery response for {host}"
+        KasaException, match=f"Timed out getting discovery response for {host}"
     ):
         await Discover.discover_single(host, discovery_timeout=0)
 
@@ -190,20 +257,21 @@ async def test_discover_invalid_info(msg, data, mocker):
     """Make sure that invalid discovery information raises an exception."""
     host = "127.0.0.1"
 
-    def mock_discover(self):
+    async def mock_discover(self):
         self.datagram_received(
             XorEncryption.encrypt(json_dumps(data))[4:], (host, 9999)
         )
 
     mocker.patch.object(_DiscoverProtocol, "do_discover", mock_discover)
 
-    with pytest.raises(SmartDeviceException, match=msg):
+    with pytest.raises(KasaException, match=msg):
         await Discover.discover_single(host)
 
 
 async def test_discover_send(mocker):
     """Test discovery parameters."""
-    proto = _DiscoverProtocol()
+    discovery_timeout = 0
+    proto = _DiscoverProtocol(discovery_timeout=discovery_timeout)
     assert proto.discovery_packets == 3
     assert proto.target_1 == ("255.255.255.255", 9999)
     transport = mocker.patch.object(proto, "transport")
@@ -232,7 +300,7 @@ async def test_discover_datagram_received(mocker, discovery_data):
     # Check that unsupported device is 1
     assert len(proto.unsupported_device_exceptions) == 1
     dev = proto.discovered_devices[addr]
-    assert issubclass(dev.__class__, SmartDevice)
+    assert issubclass(dev.__class__, Device)
     assert dev.host == addr
 
 
@@ -278,11 +346,11 @@ async def test_discover_single_authentication(discovery_mock, mocker):
     mocker.patch.object(
         device_class,
         "update",
-        side_effect=AuthenticationException("Failed to authenticate"),
+        side_effect=AuthenticationError("Failed to authenticate"),
     )
 
     with pytest.raises(
-        AuthenticationException,
+        AuthenticationError,
         match="Failed to authenticate",
     ):
         device = await Discover.discover_single(
@@ -297,23 +365,27 @@ async def test_discover_single_authentication(discovery_mock, mocker):
 
 
 @new_discovery
-async def test_device_update_from_new_discovery_info(discovery_data):
-    device = SmartDevice("127.0.0.7")
+async def test_device_update_from_new_discovery_info(discovery_mock):
+    """Make sure that new discovery devices update from discovery info correctly."""
+    discovery_data = discovery_mock.discovery_data
+    device_class = Discover._get_device_class(discovery_data)
+    device = device_class("127.0.0.1")
     discover_info = DiscoveryResult(**discovery_data["result"])
     discover_dump = discover_info.get_dict()
-    discover_dump["alias"] = "foobar"
-    discover_dump["model"] = discover_dump["device_model"]
+    model, _, _ = discover_dump["device_model"].partition("(")
+    discover_dump["model"] = model
     device.update_from_discover_info(discover_dump)
 
-    assert device.alias == "foobar"
     assert device.mac == discover_dump["mac"].replace("-", ":")
-    assert device.model == discover_dump["device_model"]
+    assert device.model == model
 
-    with pytest.raises(
-        SmartDeviceException,
-        match=re.escape("You need to await update() to access the data"),
-    ):
-        assert device.supported_modules
+    # TODO implement requires_update for SmartDevice
+    if isinstance(device, IotDevice):
+        with pytest.raises(
+            KasaException,
+            match=re.escape("You need to await update() to access the data"),
+        ):
+            assert device.supported_modules
 
 
 async def test_discover_single_http_client(discovery_mock, mocker):
@@ -323,7 +395,7 @@ async def test_discover_single_http_client(discovery_mock, mocker):
 
     http_client = aiohttp.ClientSession()
 
-    x: SmartDevice = await Discover.discover_single(host)
+    x: Device = await Discover.discover_single(host)
 
     assert x.config.uses_http == (discovery_mock.default_port == 80)
 
@@ -334,14 +406,14 @@ async def test_discover_single_http_client(discovery_mock, mocker):
 
 
 async def test_discover_http_client(discovery_mock, mocker):
-    """Make sure that discover_single returns an initialized SmartDevice instance."""
+    """Make sure that discover returns an initialized SmartDevice instance."""
     host = "127.0.0.1"
     discovery_mock.ip = host
 
     http_client = aiohttp.ClientSession()
 
     devices = await Discover.discover(discovery_timeout=0)
-    x: SmartDevice = devices[host]
+    x: Device = devices[host]
     assert x.config.uses_http == (discovery_mock.default_port == 80)
 
     if discovery_mock.default_port == 80:
@@ -402,31 +474,24 @@ class FakeDatagramTransport(asyncio.DatagramTransport):
 @pytest.mark.parametrize("port", [9999, 20002])
 @pytest.mark.parametrize("do_not_reply_count", [0, 1, 2, 3, 4])
 async def test_do_discover_drop_packets(mocker, port, do_not_reply_count):
-    """Make sure that discover_single handles authenticating devices correctly."""
+    """Make sure that _DiscoverProtocol handles authenticating devices correctly."""
     host = "127.0.0.1"
-    discovery_timeout = 1
+    discovery_timeout = 0
 
-    event = asyncio.Event()
     dp = _DiscoverProtocol(
         target=host,
         discovery_timeout=discovery_timeout,
         discovery_packets=5,
-        discovered_event=event,
     )
     ft = FakeDatagramTransport(dp, port, do_not_reply_count)
     dp.connection_made(ft)
 
-    timed_out = False
-    try:
-        async with asyncio_timeout(discovery_timeout):
-            await event.wait()
-    except asyncio.TimeoutError:
-        timed_out = True
+    await dp.wait_for_discovery_to_complete()
 
     await asyncio.sleep(0)
     assert ft.send_count == do_not_reply_count + 1
     assert dp.discover_task.done()
-    assert timed_out is False
+    assert dp.discover_task.cancelled()
 
 
 @pytest.mark.parametrize(
@@ -435,27 +500,104 @@ async def test_do_discover_drop_packets(mocker, port, do_not_reply_count):
     ids=["unknownport", "unsupporteddevice"],
 )
 async def test_do_discover_invalid(mocker, port, will_timeout):
-    """Make sure that discover_single handles authenticating devices correctly."""
+    """Make sure that _DiscoverProtocol handles invalid devices correctly."""
     host = "127.0.0.1"
-    discovery_timeout = 1
+    discovery_timeout = 0
 
-    event = asyncio.Event()
     dp = _DiscoverProtocol(
         target=host,
         discovery_timeout=discovery_timeout,
         discovery_packets=5,
-        discovered_event=event,
     )
     ft = FakeDatagramTransport(dp, port, 0, unsupported=True)
     dp.connection_made(ft)
 
-    timed_out = False
-    try:
-        async with asyncio_timeout(15):
-            await event.wait()
-    except asyncio.TimeoutError:
-        timed_out = True
-
+    await dp.wait_for_discovery_to_complete()
     await asyncio.sleep(0)
     assert dp.discover_task.done()
-    assert timed_out is will_timeout
+    assert dp.discover_task.cancelled() != will_timeout
+
+
+async def test_discover_propogates_task_exceptions(discovery_mock):
+    """Make sure that discover propogates callback exceptions."""
+    discovery_timeout = 0
+
+    async def on_discovered(dev):
+        raise KasaException("Dummy exception")
+
+    with pytest.raises(KasaException):
+        await Discover.discover(
+            discovery_timeout=discovery_timeout, on_discovered=on_discovered
+        )
+
+
+async def test_do_discover_no_connection(mocker):
+    """Make sure that if the datagram connection doesnt start a TimeoutError is raised."""
+    host = "127.0.0.1"
+    discovery_timeout = 0
+    mocker.patch.object(_DiscoverProtocol, "DISCOVERY_START_TIMEOUT", 0)
+    dp = _DiscoverProtocol(
+        target=host,
+        discovery_timeout=discovery_timeout,
+        discovery_packets=5,
+    )
+    # Normally tests would simulate connection as per below
+    # ft = FakeDatagramTransport(dp, port, 0, unsupported=True)
+    # dp.connection_made(ft)
+
+    with pytest.raises(asyncio.TimeoutError):
+        await dp.wait_for_discovery_to_complete()
+
+
+async def test_do_discover_external_cancel(mocker):
+    """Make sure that a cancel other than when target is discovered propogates."""
+    host = "127.0.0.1"
+    discovery_timeout = 1
+
+    dp = _DiscoverProtocol(
+        target=host,
+        discovery_timeout=discovery_timeout,
+        discovery_packets=1,
+    )
+    # Normally tests would simulate connection as per below
+    ft = FakeDatagramTransport(dp, 9999, 1, unsupported=True)
+    dp.connection_made(ft)
+
+    with pytest.raises(asyncio.TimeoutError):
+        async with asyncio_timeout(0):
+            await dp.wait_for_discovery_to_complete()
+
+
+async def test_discovery_redaction(discovery_mock, caplog: pytest.LogCaptureFixture):
+    """Test query sensitive info redaction."""
+    mac = "12:34:56:78:9A:BC"
+
+    if discovery_mock.default_port == 9999:
+        sysinfo = discovery_mock.discovery_data["system"]["get_sysinfo"]
+        if "mac" in sysinfo:
+            sysinfo["mac"] = mac
+        elif "mic_mac" in sysinfo:
+            sysinfo["mic_mac"] = mac
+    else:
+        discovery_mock.discovery_data["result"]["mac"] = mac
+
+    # Info no message logging
+    caplog.set_level(logging.INFO)
+    await Discover.discover()
+
+    assert mac not in caplog.text
+
+    caplog.set_level(logging.DEBUG)
+
+    # Debug no redaction
+    caplog.clear()
+    Discover._redact_data = False
+    await Discover.discover()
+    assert mac in caplog.text
+
+    # Debug redaction
+    caplog.clear()
+    Discover._redact_data = True
+    await Discover.discover()
+    assert mac not in caplog.text
+    assert "12:34:56:00:00:00" in caplog.text
